@@ -2,6 +2,7 @@ package XMLTV::Options;
 
 use strict;
 use warnings;
+use vars qw($fd);
 
 BEGIN {
     use Exporter   ();
@@ -67,8 +68,11 @@ my %cap_options = (
                              cache:s
                              /],
 		   share => [qw/
-			     share:se
+			     share=s
 			     /],
+		   preferredmethod => [qw/
+				       preferredmethod
+				       /],
 		   );
 
 my %cap_defaults = (
@@ -101,10 +105,14 @@ my %cap_defaults = (
 		    share => {
 			share => undef,
 		    },
+		    preferredmethod => {
+			preferredmethod => 0,
+		    },
 		    );
 
+=head1 USAGE
 
-=item ParseOptions
+=item B<ParseOptions>
 
 ParseOptions shall be called by a grabber to parse the command-line
 options supplied by the user. It takes a single hashref as a parameter.
@@ -112,7 +120,7 @@ The entries in the hash configure the behaviour of ParseOptions.
 
   my( $opt, $conf ) = ParseOptions( {
     grabber_name => 'tv_grab_test',
-    version => '$Id: Options.pm,v 1.11 2006/06/03 19:36:52 epaepa Exp $',
+    version => '$Id: Options.pm,v 1.15 2008/06/01 01:48:00 knowledgejunkie Exp $',
     description => 'Sweden (tv.swedb.se)',
     capabilities => [qw/baseline manualconfig apiconfig/],
     stage_sub => \&config_stage,
@@ -149,6 +157,11 @@ ParseOptions handles the following options automatically without returning:
 
 =item --description
 
+=item --preferredmethod 
+
+Handled automatically if the preferredmethod capability has been set and
+the preferredmethod option has been specified in the call to ParseOptions.
+ 
 =back
 
 ParseOptions also takes care of the following options without returning,
@@ -174,15 +187,24 @@ If the --output option is specified, STDOUT will be redirected to
 the specified file.
 
 The grabber must check the following options on its own:
---days
---offset
---quiet
---debug
+
+=over
+
+=item --days
+
+=item --offset
+
+=item --quiet
+
+=item --debug
+
+=back
 
 and any other options that are grabber specific. This can be done by reading
 $opt->{days} etc.
 
-Changing the behaviour of ParseOptions
+=item B<Changing the behaviour of ParseOptions>
+
 
 The behaviour of ParseOptions can be influenced by passing named arguments
 in the hashref. The following arguments are supported:
@@ -198,8 +220,8 @@ when printing the synopsis.
 
 Required. The description for the grabber. This is returned in response to
 the --description option and shall say which region the grabber returns data
-for (e.g. "Sweden" or "Sweden (tv.swedb.se)" if there are several grabbers
-for a country).
+for. Examples: "Sweden", or "Sweden (tv.swedb.se)" if there are several grabbers
+for a region or country).
 
 =item version
 
@@ -208,11 +230,11 @@ Required. This shall be a cvs Id field.
 =item capabilities
 
 Required. The capabilities that the grabber shall support. Only capabilities
-that XMLTV::Options know how to handle can be specified. Example:
+that XMLTV::Options knows how to handle can be specified. Example:
 
   capabilities => [qw/baseline manualconfig apiconfig/],
 
-Note that XMLTV::Options guarantee that the grabber supports the manualconfig
+Note that XMLTV::Options guarantees that the grabber supports the manualconfig
 and apiconfig capabilities. The capabilities share and cache can be
 specified if the grabber supports them. XMLTV::Options will then automatically
 accept the command-line parameters --share and --cache respectively.
@@ -226,10 +248,10 @@ The xml-string shall follow the xmltv-configuration.dtd.
 
 =item listchannels_sub
 
-Required. listchannels_sub shall be a coderef that takes a configuration
+Required. A coderef that takes a configuration
 hash as returned by XMLTV::Configure::LoadConfig as the first parameter
 and an option hash as returned by
-ParseOptions as the second parameter and returns an xml-string
+ParseOptions as the second parameter, and returns an xml-string
 containing a list of all the channels that the grabber can deliver
 data for using the supplied configuration. Note that the listsub
 shall not use any channel-configuration from the hashref.
@@ -243,6 +265,21 @@ XMLTV::Configure::LoadConfig fails to parse the configuration file. This
 allows the grabber to load configuration files created with an older
 version of the grabber.
 
+=item preferredmethod
+
+Optional. A value to return when the grabber is called with the 
+--preferredmethod parameter. Example:
+
+  my( $opt, $conf ) = ParseOptions( {
+    grabber_name => 'tv_grab_test',
+    version => '$Id: Options.pm,v 1.15 2008/06/01 01:48:00 knowledgejunkie Exp $',
+    description => 'Sweden (tv.swedb.se)',
+    capabilities => [qw/baseline manualconfig apiconfig preferredmethod/],
+    stage_sub => \&config_stage,
+    listchannels_sub => \&list_channels,
+    preferredmethod => 'allatonce',
+  } );
+
 =item defaults
 
 Optional. Default {}. A hashref that contains default values for the
@@ -265,6 +302,12 @@ sub ParseOptions
 
     my @optdef=();
     my $opt={};
+    my $capabilities = {};
+
+    foreach my $cap (keys %{cap_options}) 
+    {
+	$capabilities->{$cap} = 0;
+    }
 
     if( not defined( $p->{version} ) )
     {
@@ -276,6 +319,7 @@ sub ParseOptions
 	croak "No description specified in call to ParseOptions";
     }
 
+
     push( @optdef, @{$cap_options{all}} );
     hash_push( $opt, $cap_defaults{all} );
 
@@ -292,6 +336,20 @@ sub ParseOptions
 	
 	push( @optdef, @{$cap_options{$cap}} );
 	hash_push( $opt, $cap_defaults{$cap} );
+	$capabilities->{$cap} = 1;
+    }
+
+    if( $capabilities->{preferredmethod} 
+	and not exists($p->{preferredmethod}) )
+    {
+	croak "You must specify which preferredmethod to use";
+    }
+
+    if( exists($p->{preferredmethod}) 
+	and not $capabilities->{preferredmethod} )
+    {
+	croak "You must include the capability preferredmethod to specify " .
+	    "which preferredmethod to use.";
     }
 
     push( @optdef, @{$p->{extra_options}} )
@@ -312,6 +370,11 @@ sub ParseOptions
 	print join( "\n", @{$p->{capabilities}} ) . "\n";
 	exit 0;
     }
+    elsif( $opt->{preferredmethod} )
+    {
+	print $p->{preferredmethod} . "\n";
+	exit 0;
+    }
     elsif( $opt->{version} )
     {
 	eval {
@@ -341,14 +404,14 @@ sub ParseOptions
 
     if( defined( $opt->{output} ) )
     {
-	if( not open( OUT, "> $opt->{output}" ) )
+	if( not open( $fd, "> $opt->{output}" ) )
 	{
 	    print STDERR "Cannot write to $opt->{output}.";
 	    exit 1;
 	}
 	
 	# Redirect STDOUT to the file.
-	select( OUT );
+	select( $fd );
     }
 
     if( $opt->{configure} )
